pub type TetriminoData = &'static [&'static [bool]];

// these particular letter choices make it easier to see the shape of the tetrimino
#[allow(non_upper_case_globals)]
const o: bool = false;

#[allow(non_upper_case_globals)]
const U: bool = true;

pub const I: [TetriminoData; 4] = [
    &[
        &[o, o, o, o],
        &[U, U, U, U],
        &[o, o, o, o],
        &[o, o, o, o],
    ],
    &[
        &[o, o, U, o],
        &[o, o, U, o],
        &[o, o, U, o],
        &[o, o, U, o],
    ],
    &[
        &[o, o, o, o],
        &[o, o, o, o],
        &[U, U, U, U],
        &[o, o, o, o],
    ],
    &[
        &[o, U, o, o],
        &[o, U, o, o],
        &[o, U, o, o],
        &[o, U, o, o],
    ],
];

pub const O: [TetriminoData; 4] = [
    &[
        &[o, U, U, o],
        &[o, U, U, o],
    ],
    &[
        &[o, U, U, o],
        &[o, U, U, o],
    ],
    &[
        &[o, U, U, o],
        &[o, U, U, o],
    ],
    &[
        &[o, U, U, o],
        &[o, U, U, o],
    ],
];

pub const T: [TetriminoData; 4] = [
    &[
        &[o, U, o],
        &[U, U, U],
        &[o, o, o],
    ],
    &[
        &[o, U, o],
        &[o, U, U],
        &[o, U, o],
    ],
    &[
        &[o, o, o],
        &[U, U, U],
        &[o, U, o],
    ],
    &[
        &[o, U, o],
        &[U, U, o],
        &[o, U, o],
    ],
];

pub const S: [TetriminoData; 4] = [
    &[
        &[o, U, U],
        &[U, U, o],
        &[o, o, o],
    ],
    &[
        &[o, U, o],
        &[o, U, U],
        &[o, o, U],
    ],
    &[
        &[o, o, o],
        &[o, U, U],
        &[U, U, o],
    ],
    &[
        &[U, o, o],
        &[U, U, o],
        &[o, U, o],
    ],
];

pub const Z: [TetriminoData; 4] = [
    &[
        &[U, U, o],
        &[o, U, U],
        &[o, o, o],
    ],
    &[
        &[o, o, U],
        &[o, U, U],
        &[o, U, o],
    ],
    &[
        &[o, o, o],
        &[U, U, o],
        &[o, U, U],
    ],
    &[
        &[o, U, o],
        &[U, U, o],
        &[U, o, o],
    ],
];

pub const J: [TetriminoData; 4] = [
    &[
        &[U, o, o],
        &[U, U, U],
        &[o, o, o],
    ],
    &[
        &[o, U, U],
        &[o, U, o],
        &[o, U, o],
    ],
    &[
        &[o, o, o],
        &[U, U, U],
        &[o, o, U],
    ],
    &[
        &[o, U, o],
        &[o, U, o],
        &[U, U, o],
    ],
];

pub const L: [TetriminoData; 4] = [
    &[
        &[o, o, U],
        &[U, U, U],
        &[o, o, o],
    ],
    &[
        &[o, U, o],
        &[o, U, o],
        &[o, U, U],
    ],
    &[
        &[o, o, o],
        &[U, U, U],
        &[U, o, o],
    ],
    &[
        &[U, U, o],
        &[o, U, o],
        &[o, U, o],
    ],
];


#[derive(Debug, Clone, Copy)]
#[repr(u8)]
pub enum Rotation {
    Spawn,
    Clockwise,
    Flipped,
    CounterClockwise 
}

#[derive(Debug, Clone, Copy)]
pub struct WallkickTable([[(i32, i32); 5]; 8]);

impl WallkickTable {
    pub fn for_rotation_cw(self, rotation: Rotation) -> [(i32, i32); 5] {
        match rotation {
            Rotation::Spawn => self.0[0],
            Rotation::Clockwise => self.0[2],
            Rotation::Flipped => self.0[4],
            Rotation::CounterClockwise => self.0[6],
        }
    }

    pub fn for_rotation_ccw(self, rotation: Rotation) -> [(i32, i32); 5] {
        match rotation {
            Rotation::Spawn => self.0[7],
            Rotation::Clockwise => self.0[1],
            Rotation::Flipped => self.0[3],
            Rotation::CounterClockwise => self.0[5],
        }
    }
}

pub const WALLKICK_TABLE_1: WallkickTable = WallkickTable([
    [( 0, 0), (-1, 0), (-1, 1), ( 0,-2), (-1,-2)], // 0>>1
    [( 0, 0), ( 1, 0), ( 1,-1), ( 0, 2), ( 1, 2)], // 1>>0
    [( 0, 0), ( 1, 0), ( 1,-1), ( 0, 2), ( 1, 2)], // 1>>2
    [( 0, 0), (-1, 0), (-1, 1), ( 0,-2), (-1,-2)], // 2>>1
    [( 0, 0), ( 1, 0), ( 1, 1), ( 0,-2), ( 1,-2)], // 2>>3
    [( 0, 0), (-1, 0), (-1,-1), ( 0, 2), (-1, 2)], // 3>>2
    [( 0, 0), (-1, 0), (-1,-1), ( 0, 2), (-1, 2)], // 3>>0
    [( 0, 0), ( 1, 0), ( 1, 1), ( 0,-2), ( 1,-2)], // 0>>3
]);

pub const WALLKICK_TABLE_2: WallkickTable = WallkickTable([
    [( 0, 0), (-2, 0), ( 1, 0), (-2,-1), ( 1, 2)], // 0>>1
    [( 0, 0), ( 2, 0), (-1, 0), ( 2, 1), (-1,-2)], // 1>>0
    [( 0, 0), (-1, 0), ( 2, 0), (-1, 2), ( 2,-1)], // 1>>2
    [( 0, 0), ( 1, 0), (-2, 0), ( 1,-2), (-2, 1)], // 2>>1
    [( 0, 0), ( 2, 0), (-1, 0), ( 2, 1), (-1,-2)], // 2>>3
    [( 0, 0), (-2, 0), ( 1, 0), (-2,-1), ( 1, 2)], // 3>>2
    [( 0, 0), ( 1, 0), (-2, 0), ( 1,-2), (-2, 1)], // 3>>0
    [( 0, 0), (-1, 0), ( 2, 0), (-1, 2), ( 2,-1)], // 0>>3
]);